<?php

namespace Drupal\Tests\domain_group\Functional;

use Drupal\Tests\domain_group\Traits\GroupCreationTrait;
use Drupal\Tests\domain_group\Traits\InitializeGroupsTrait;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the group and group content access.
 *
 * @group domain_group
 */
class UniqueGroupAccessTest extends BrowserTestBase {

  use GroupCreationTrait;
  use InitializeGroupsTrait;

  /**
   * Will be removed when issue #3204455 on Domain Site Settings gets merged.
   *
   * See https://www.drupal.org/project/domain_site_settings/issues/3204455.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Testing\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'node',
    'block',
    'group',
    'gnode',
    'domain',
    'domain_site_settings',
    'domain_group',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'classy';

  /**
   * Regular authenticated User for tests.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $testUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Setup the group types and test groups from the InitializeGroupsTrait.
    $this->initializeTestGroups();
    $this->initializeTestGroupsDomains();
    $this->initializeTestGroupContent();

    // Allow anonymous to view groups of type A.
    $this->groupTypeA->getAnonymousRole()->grantPermissions([
      'view group',
    ])->save();
    
    // Allow outsider to view group content article of type A.
    $this->groupTypeA->getOutsiderRole()->grantPermissions([
      'view group_node:article entity',
    ])->save();

    // Create test user.
    $this->testUser = $this->drupalCreateUser([
      'access content',
    ]);
  }

  /**
   * Test group access when unique group access is enabled.
   */
  public function testUniqueGroupAccess() {
    $domain_storage = \Drupal::entityTypeManager()->getStorage('domain');

    // Visiting groups from default domain.
    $this->drupalGet('group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(200);

    $ga1_domain = $domain_storage->load('group_' . $this->groupA1->id());
    // Visiting group A1 page.
    $this->drupalGet($ga1_domain->getPath());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA3->id());
    $this->assertSession()->statusCodeEquals(403);

    $ga2_domain = $domain_storage->load('group_' . $this->groupA2->id());
    // Visiting group A2 page.
    $this->drupalGet($ga2_domain->getPath());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups should be denied.
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA3->id());
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Test group access when unique group access is disabled.
   */
  public function testUniqueGroupAccessDisabled() {
    $domain_storage = \Drupal::entityTypeManager()->getStorage('domain');

    // Disable unique group access.
    $this->setUniqueGroupAccess(FALSE);

    // Visiting groups from default domain.
    $this->drupalGet('group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(200);

    $ga1_domain = $domain_storage->load('group_' . $this->groupA1->id());
    // Visiting other groups from group domain should be allowed.
    $this->drupalGet($ga1_domain->getPath());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga1_domain->getPath() . '/group/' . $this->groupA3->id());
    $this->assertSession()->statusCodeEquals(200);

    $ga2_domain = $domain_storage->load('group_' . $this->groupA2->id());
    // Visiting other groups from group domain should be allowed.
    $this->drupalGet($ga2_domain->getPath());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA2->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga2_domain->getPath() . '/group/' . $this->groupA3->id());
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Test content access when unique group access is enabled.
   */
  public function testUniqueContentAccess() {
    $this->drupalLogin($this->testUser);
    $domain_storage = \Drupal::entityTypeManager()->getStorage('domain');

    // Visiting groups from default domain.
    $this->drupalGet('node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(200);
    // Editing should be denied.
    $this->drupalGet('node/' . $this->nodeA1->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('node/' . $this->nodeA2->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    // Deleting should be denied.
    $this->drupalGet('node/' . $this->nodeA1->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('node/' . $this->nodeA2->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);

    $ga1_domain = $domain_storage->load('group_' . $this->groupA1->id());
    // Visiting group A1 content.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups' content should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id());
    $this->assertSession()->statusCodeEquals(403);
    // Editing should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    // Deleting should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);

    $ga2_domain = $domain_storage->load('group_' . $this->groupA2->id());
    // Visiting group A2 content.
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups' content should be denied.
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA3->id());
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Test content access when unique group access is disabled.
   */
  public function testUniqueContentAccessDisabled() {
    $this->drupalLogin($this->testUser);
    $domain_storage = \Drupal::entityTypeManager()->getStorage('domain');
    // Disable unique group access.
    $this->setUniqueGroupAccess(FALSE);

    // Visiting groups from default domain.
    $this->drupalGet('node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(200);
    // Editing should be denied.
    $this->drupalGet('node/' . $this->nodeA1->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('node/' . $this->nodeA2->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    // Deleting should be denied.
    $this->drupalGet('node/' . $this->nodeA1->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('node/' . $this->nodeA2->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);

    $ga1_domain = $domain_storage->load('group_' . $this->groupA1->id());
    // Visiting group A1 content.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups' content should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id());
    $this->assertSession()->statusCodeEquals(200);
    // Editing should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id() . '/edit');
    $this->assertSession()->statusCodeEquals(403);
    // Deleting should be denied.
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA1->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA2->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet($ga1_domain->getPath() . '/node/' . $this->nodeA3->id() . '/delete');
    $this->assertSession()->statusCodeEquals(403);

    $ga2_domain = $domain_storage->load('group_' . $this->groupA2->id());
    // Visiting group A2 content.
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA2->id());
    $this->assertSession()->statusCodeEquals(200);
    // Visiting other groups' content should be denied.
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA1->id());
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet($ga2_domain->getPath() . '/node/' . $this->nodeA3->id());
    $this->assertSession()->statusCodeEquals(200);
  }

}
